Overview — SQLAlchemy 1.4 Documentation

Release: 1.4.32 | Release Date: March 6, 2022

# SQLAlchemy 1.4 Documentation

# Overview

The SQLAlchemy SQL Toolkit and Object Relational Mapper is a comprehensive set of tools for working with databases and Python. It has several distinct areas of functionality which can be used individually or combined together. Its major components are illustrated below, with component dependencies organized into layers:

Above, the two most significant front-facing portions of SQLAlchemy are the Object Relational Mapper (ORM) and the Core.

Core contains the breadth of SQLAlchemy’s SQL and database integration and description services, the most prominent part of this being the SQL Expression Language.

The SQL Expression Language is a toolkit all its own, independent of the ORM package, which provides a system of constructing SQL expressions represented by composable objects, which can then be “executed” against a target database within the scope of a specific transaction, returning a result set. Inserts, updates and deletes (i.e. DML) are achieved by passing SQL expression objects representing these statements along with dictionaries that represent parameters to be used with each statement.

The ORM builds upon Core to provide a means of working with a domain object model mapped to a database schema. When using the ORM, SQL statements are constructed in mostly the same way as when using Core, however the task of DML, which here refers to the persistence of business objects in a database, is automated using a pattern called unit of work, which translates changes in state against mutable objects into INSERT, UPDATE and DELETE constructs which are then invoked in terms of those objects. SELECT statements are also augmented by ORM-specific automations and object-centric querying capabilities.

Whereas working with Core and the SQL Expression language presents a schema-centric view of the database, along with a programming paradigm that is oriented around immutability, the ORM builds on top of this a domain-centric view of the database with a programming paradigm that is more explcitly object-oriented and reliant upon mutability. Since a relational database is itself a mutable service, the difference is that Core/SQL Expression language is command oriented whereas the ORM is state oriented.

## Documentation Overview

The documentation is separated into four sections:

- SQLAlchemy 1.4 / 2.0 Tutorial - this all-new tutorial for the 1.4/2.0 series of SQLAlchemy introduces the entire library holistically, starting from a description of Core and working more and more towards ORM-specific concepts. New users, as well as users coming from 1.x style, who wish to work in 2.0 style should start here.

- SQLAlchemy ORM - In this section, reference documentation for the ORM is presented; this section also includes the now-legacy Object Relational Tutorial (1.x API).

- SQLAlchemy Core - Here, reference documentation for everything else within Core is presented; section also includes the legacy SQL Expression Language Tutorial (1.x API). SQLAlchemy engine, connection, and pooling services are also described here.

- Dialects - Provides reference documentation for all dialect implementations, including DBAPI specifics.

## Code Examples

Working code examples, mostly regarding the ORM, are included in the SQLAlchemy distribution. A description of all the included example applications is at ORM Examples.

There is also a wide variety of examples involving both core SQLAlchemy constructs as well as the ORM on the wiki. See Theatrum Chemicum.

## Installation Guide

### Supported Platforms

SQLAlchemy has been tested against the following platforms:

- cPython 2.7

- cPython 3.6 and higher

- PyPy 2.1 or greater

Changed in version 1.4: Within the Python 3 series, 3.6 is now the minimum Python 3 version supported.

See also

Python 3.6 is the minimum Python 3 version; Python 2.7 still supported

### AsyncIO Support

SQLAlchemy’s `asyncio` support depends upon the greenlet project. This dependency will be installed by default on common machine platforms, however is not supported on every architecture and also may not install by default on less common architectures. See the section Asyncio Platform Installation Notes for additional details on ensuring asyncio support is present.

### Supported Installation Methods

SQLAlchemy installation is via standard Python methodologies that are based on setuptools, either by referring to `setup.py` directly or by using pip or other setuptools-compatible approaches.

Changed in version 1.1: setuptools is now required by the setup.py file; plain distutils installs are no longer supported.

### Install via pip

When `pip` is available, the distribution can be downloaded from PyPI and installed in one step:

    pip install SQLAlchemy

This command will download the latest released version of SQLAlchemy from the Python Cheese Shop and install it to your system.

In order to install the latest prerelease version, such as `1.4.0b1`, pip requires that the `--pre` flag be used:

    pip install --pre SQLAlchemy

Where above, if the most recent version is a prerelease, it will be installed instead of the latest released version.

### Installing using setup.py

Otherwise, you can install from the distribution using the `setup.py` script:

    python setup.py install

### Installing the C Extensions

SQLAlchemy includes C extensions which provide an extra speed boost for dealing with result sets. The extensions are supported on both the 2.xx and 3.xx series of cPython.

`setup.py` will automatically build the extensions if an appropriate platform is detected. If the build of the C extensions fails due to a missing compiler or other issue, the setup process will output a warning message and re-run the build without the C extensions upon completion, reporting final status.

To run the build/install without even attempting to compile the C extensions, the `DISABLE_SQLALCHEMY_CEXT` environment variable may be specified. The use case for this is either for special testing circumstances, or in the rare case of compatibility/build issues not overcome by the usual “rebuild” mechanism:

    export DISABLE_SQLALCHEMY_CEXT=1; python setup.py install

Changed in version 1.1: The legacy `--without-cextensions` flag has been removed from the installer as it relies on deprecated features of setuptools.

### Installing a Database API

SQLAlchemy is designed to operate with a DBAPI implementation built for a particular database, and includes support for the most popular databases. The individual database sections in Dialects enumerate the available DBAPIs for each database, including external links.

### Checking the Installed SQLAlchemy Version

This documentation covers SQLAlchemy version 1.4. If you’re working on a system that already has SQLAlchemy installed, check the version from your Python prompt like this:

    >>> import sqlalchemy
    >>> sqlalchemy.__version__
    1.4.0

### Next Steps

With SQLAlchemy installed, new and old users alike can Proceed to the SQLAlchemy Tutorial.

## 1.3 to 1.4 Migration

Notes on what’s changed from 1.3 to 1.4 is available here at What’s New in SQLAlchemy 1.4?.